<?php

namespace LakeDrops\DrupalEnvironment;

use Composer\Composer;
use Composer\Installers\Installer;
use Composer\IO\IOInterface;
use Composer\Plugin\Capability\CommandProvider as ComposerCommandProvider;
use Composer\Script\Event;
use Composer\Script\ScriptEvents;
use Drupal\Composer\Plugin\Scaffold\Handler as DrupalScaffoldHandler;
use LakeDrops\Component\Composer\BasePlugin;

/**
 * Composer plugin for handling drupal scaffold.
 */
class Plugin extends BasePlugin {

  /**
   * The scaffold handler object for events.
   *
   * @var \Drupal\Composer\Plugin\Scaffold\Handler
   */
  protected DrupalScaffoldHandler $scaffoldHandler;

  /**
   * The installer.
   *
   * @var Installer
   */
  private Installer $installer;

  /**
   * {@inheritdoc}
   */
  public function activate(Composer $composer, IOInterface $io): void {
    parent::activate($composer, $io);
    $this->installer = new Installer($io, $composer);
    $composer->getInstallationManager()->addInstaller($this->installer);
    if (class_exists(DrupalScaffoldHandler::class)) {
      $this->scaffoldHandler = new DrupalScaffoldHandler($composer, $io);
    }
    $extra = $composer->getPackage()->getExtra();
    if (isset($extra['lakedrops-config-file']) || isset($extra['lakedrops-curated-file'])) {
      if (isset($extra['lakedrops-config-file'])) {
        foreach ($this->readJson('config', $extra['lakedrops-config-file'], $io) as $key => $value) {
          $composer->getConfig()
            ->getConfigSource()
            ->addConfigSetting($key, $value);
        }
      }
      if (isset($extra['lakedrops-curated-file'])) {
        $audit = $composer->getConfig()->get('audit');
        $audit['ignore'] = $this->readJson('audit ignore', $extra['lakedrops-curated-file'], $io);
        $composer->getConfig()
          ->getConfigSource()
          ->addConfigSetting('audit', $audit);
      }
    }

    $extras = [
      'drupal-libraries',
      'drupal-scaffold',
      'installer-types',
      'installer-paths',
    ];
    $changed = FALSE;
    foreach ($extras as $item) {
      $key = 'lakedrops-extra-' . $item . '-file';
      if (isset($extra[$key])) {
        $extra[$item] = $this->readJson($item, $extra[$key], $io);
        $changed = TRUE;
      }
    }
    if ($changed) {
      $composer->getPackage()->setExtra($extra);
    }
  }

  /**
   * {@inheritdoc}
   */
  public function deactivate(Composer $composer, IOInterface $io): void {
    parent::deactivate($composer, $io);
    $composer->getInstallationManager()->removeInstaller($this->installer);
  }

  /**
   * {@inheritdoc}
   */
  public function getHandlerClass(): string {
    return Handler::class;
  }

  /**
   * {@inheritdoc}
   */
  public function getCapabilities(): array {
    return [
      ComposerCommandProvider::class => CommandProvider::class,
    ];
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array {
    return [
      ScriptEvents::POST_CREATE_PROJECT_CMD => ['prepareProject', 1],
      ScriptEvents::POST_INSTALL_CMD => ['prepareProject', 1],
    ];
  }

  /**
   * Post update project event callback.
   *
   * @param \Composer\Script\Event $event
   *   The event that triggered the plugin.
   */
  public function prepareProject(Event $event): void {
    /** @var \LakeDrops\DrupalEnvironment\Handler $handler */
    $handler = $this->handler;
    $handler->setEvent($event);
    $handler->setupLakeDropsProject();
  }

  /**
   * Helper function to read and decode default settings from a file.
   *
   * @param string $type
   *   The type of information for status message.
   * @param string $filename
   *   The filename from where to read the data.
   * @param \Composer\IO\IOInterface $io
   *   The io service for messages.
   *
   * @return array
   *   The read data.
   *
   * @throws \Exception
   */
  protected function readJson(string $type, string $filename, IOInterface $io): array {
    $io->write('<info>Gathering ' . $type . ' from patch file.</info>');
    try {
      $data = json_decode(file_get_contents($filename), TRUE, 512, JSON_THROW_ON_ERROR);
      $error = json_last_error();
    } catch (\JsonException) {
      $error = JSON_ERROR_SYNTAX;
    }
    if ($error !== 0) {
      $msg = match ($error) {
        JSON_ERROR_DEPTH => ' - Maximum stack depth exceeded',
        JSON_ERROR_STATE_MISMATCH => ' - Underflow or the modes mismatch',
        JSON_ERROR_CTRL_CHAR => ' - Unexpected control character found',
        JSON_ERROR_SYNTAX => ' - Syntax error, malformed JSON',
        JSON_ERROR_UTF8 => ' - Malformed UTF-8 characters, possibly incorrectly encoded',
        default => ' - Unknown error',
      };
      throw new \Exception('There was an error in the supplied file:' . $msg);
    }
    return $data;
  }

}
